# (c) 2023-present Eroax
# (c) 2023-present Yagich
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
extends DeckNode


func _init() -> void:
	name = "Add Vectors"
	node_type = "vector_add"
	description = "Adds two 2D vectors."
	category = "math"

	add_input_port(
		DeckType.Types.DICTIONARY,
		"Vector A"
	)
	add_input_port(
		DeckType.Types.DICTIONARY,
		"Vector B"
	)
	add_output_port(
		DeckType.Types.DICTIONARY,
		"Result"
	)


func _value_request(_on_port: int) -> Variant:
	var va = request_value(0)
	var vb = request_value(1)

	if !va || !vb:
		DeckHolder.logger.log_node("Vector Add: one of the vectors is invalid.", Logger.LogType.ERROR)
		return null

	if !(va as Dictionary).has("x") || !(va as Dictionary).has("y"):
		DeckHolder.logger.log_node("Vector Add: one of the vectors is invalid.", Logger.LogType.ERROR)
		return null

	if !(vb as Dictionary).has("x") || !(vb as Dictionary).has("y"):
		DeckHolder.logger.log_node("Vector Add: one of the vectors is invalid.", Logger.LogType.ERROR)
		return null

	var res := {}
	res["x"] = va["x"] + vb["x"]
	res["y"] = va["y"] + vb["y"]

	return res
