# (c) 2023-present Eroax
# (c) 2023-present Yagich
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
extends DeckNode


func _init() -> void:
	name = "Multiply Vector by Scalar"
	node_type = "vector_multiply"
	description = "Multiplies a vector by a numeric value."
	category = "math"

	add_input_port(
		DeckType.Types.DICTIONARY,
		"Vector"
	)
	add_input_port(
		DeckType.Types.NUMERIC,
		"Scalar",
		"field"
	)

	add_output_port(
		DeckType.Types.DICTIONARY,
		"Result"
	)


func _value_request(_on_port: int) -> Variant:
	var v = request_value(0)

	if !v:
		DeckHolder.logger.log_node("Vector Mult: the vector is invalid.", Logger.LogType.ERROR)
		return null

	if !(v as Dictionary).has("x") || !(v as Dictionary).has("y"):
		DeckHolder.logger.log_node("Vector Mult: the vector is invalid.", Logger.LogType.ERROR)
		return null

	var s = float(resolve_input_port_value(1))

	return {"x": v.x * s, "y": v.y * s}
